
#include <qcstring.h>
#include <qfile.h>
#include <qregexp.h>

#include <stdlib.h>

#include <kapplication.h>
#include <kdebug.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <klocale.h>
#include <kurl.h>
#include <kio/jobclasses.h>
#include <kio/job.h>
#include <kprocess.h>
#include <kconfig.h>
#include <kdirsize.h>
#include <dcopclient.h>
#include <qbuttongroup.h>
#include <kpushbutton.h>
#include <qcheckbox.h>
#include <qtimer.h>
#include <kwizard.h>
#include <kwin.h>
#include <klistview.h>
#include <qlabel.h>
#include <kcombobox.h>
#include <qfile.h>

#include "burn.h"
#include "progressdlg.h"
#include "driveselectpage.h"
#include "burndiscwizard.h"
#include "burnoptionspage.h"
#include "advancedsettingsdialog.h"
#include "settingswizard.h"

using namespace KIO;

/*UDS Utility functions - borrowed from audiocd kioslave*/
static void
    app_entry(UDSEntry& e, unsigned int uds, const QString& str)
{
  UDSAtom a;
  a.m_uds = uds;
  a.m_str = str;
  e.append(a);
}

static void
    app_entry(UDSEntry& e, unsigned int uds, long l)
{
  UDSAtom a;
  a.m_uds = uds;
  a.m_long = l;
  e.append(a);
}

static void
    app_dir(UDSEntry& e, const QString & n, size_t s, long access)
{
  e.clear();
  app_entry(e, KIO::UDS_NAME, QFile::decodeName(n.local8Bit()));
  app_entry(e, KIO::UDS_FILE_TYPE, S_IFDIR);
  app_entry(e, KIO::UDS_ACCESS, access);
  app_entry(e, KIO::UDS_SIZE, s);
}

static void
    app_file(UDSEntry& e, const QString & n, size_t s, long access)
{
  e.clear();
  app_entry(e, KIO::UDS_NAME, QFile::decodeName(n.local8Bit()));
  app_entry(e, KIO::UDS_FILE_TYPE, S_IFREG);
  app_entry(e, KIO::UDS_ACCESS, access);
  app_entry(e, KIO::UDS_SIZE, s);
}

/******************** UTITILITY FUNCTIONS *************************/


void kio_burnProtocol::error_user_cancel( )
{
  error( KIO::ERR_USER_CANCELED, "" );
  finished();
}
    
  /***************************CONSTRUCTOR*****************/
  
kio_burnProtocol::kio_burnProtocol(const QCString &pool_socket, const QCString &app_socket)
  : QObject(), SlaveBase("kio_burn", pool_socket, app_socket)
{
  m_data_dir = new QString( locateLocal( "data", "kio_burn/data/" ) );
  m_audio_dir = new QString( locateLocal( "data", "kio_burn/audio/" ) );
  
  m_data_rx = new QRegExp( QString("^/?(") += i18n("Data") += QString("|") += i18n("Audio CD")  += ")/?"  );
  m_data_rxx = new QRegExp( QString("^/?(") += i18n("Data") += QString("|") += i18n("Audio CD") += ")/"  );
  
  m_config = new KConfig( "kio_burnrc", false, false, "config" );
  
  m_cdrecordLog = new QFile( locateLocal( "data", "kio_burn/cdrecordlog" ) );
  
}


kio_burnProtocol::~kio_burnProtocol()
{
  delete m_config;
  delete m_data_rx;
  delete m_data_rxx;
  delete m_data_dir;
  delete m_audio_dir;
  delete m_cdrecordLog;
}

/**************************IO SLAVE FUNCTIONS*********************************/

void kio_burnProtocol::get(const KURL& url )
  {
    
    if( url.path().find( QString( "/" ).append( i18n( "Burn Disc" ) ) ) > -1 ){

            
      m_burnDiscWizard = new BurnDiscWizard(m_config);
      m_burnDiscWizard->show();
      
      KWin::raiseWindow( m_burnDiscWizard->winId() ); //shouldn't really be using this function but if we don't then
                                                //konq takes our focus
      KWin::forceActiveWindow( m_burnDiscWizard->winId() );
            
      connect( m_burnDiscWizard->finishButton(), SIGNAL( clicked() ),
               this, SLOT( exitQAppLoop() ) );
      connect( m_burnDiscWizard->cancelButton(), SIGNAL( clicked() ),
               this, SLOT( exitQAppLoop() ) );
      qApp->enter_loop();
      
      //save any settings that we selected in the burn disc dialog
      m_config->writeEntry( "disctype", 
                            m_burnDiscWizard->optionsPage()->discTypeGroup->selectedId() );
     // m_config->writeEntry( "blankcdrw", 
     //                       m_burnDiscWizard->optionsPage()->blankCheckBox->isChecked() );
     // m_config->writeEntry( "blankonly", 
     //                       m_burnDiscWizard->optionsPage()->blankOnlyCheckBox->isChecked() );
      m_config->writeEntry( "advsettings",
                            m_burnDiscWizard->advancedSettingsDialog()->advancedSettingsCheckBox->isChecked() );
      m_config->writeEntry( "nofix",
                            m_burnDiscWizard->advancedSettingsDialog()->noFixateCheckBox->isChecked() );
      m_config->writeEntry( "burnmode", 
                            m_burnDiscWizard->advancedSettingsDialog()->burnModeCombo->currentItem() );
      if( m_burnDiscWizard->advancedSettingsDialog()->burnSpeedCombo->currentText() != i18n( "Scanning..." ) ) //don't save the burning speed if we didn't finish scanning it
        m_config->writeEntry( "burnspeed",
                              m_burnDiscWizard->advancedSettingsDialog()->burnSpeedCombo->currentText() );
        
      m_config->sync();
      
      if(  m_burnDiscWizard->result() == QDialog::Accepted  )
        {
          //look for a DCOP client to send progress information to
          //this should be the kio_burn plugin in konqueror
          m_dcopId = url.queryItem( "dcopid" ) ;
          
          m_options.blankCDRW = m_burnDiscWizard->mediaIsRW();
          //m_options.blankOnly = m_burnDiscWizard->optionsPage()->blankOnlyCheckBox->isChecked();
          m_options.advSettings = m_burnDiscWizard->advancedSettingsDialog()->advancedSettingsCheckBox->isChecked();
          m_options.doNotFixate = m_burnDiscWizard->advancedSettingsDialog()->noFixateCheckBox->isChecked();
          
          if( m_burnDiscWizard->advancedSettingsDialog()->burnSpeedCombo->currentText() != i18n( "Scanning..." ) )
            m_options.burnSpeed = m_burnDiscWizard->advancedSettingsDialog()->burnSpeedCombo->currentText();
          else
            m_options.burnSpeed = m_config->readEntry( "burnspeed", 0L );
          
          switch( m_burnDiscWizard->advancedSettingsDialog()->burnModeCombo->currentItem() )
          {
            case 0:
              m_options.mode = BurnOptions::TAO;
              break;
            case 1:
              m_options.mode = BurnOptions::SAO;
              break;
          }
          switch( m_burnDiscWizard->optionsPage()->discTypeGroup->selectedId()  )
          {
            case 0:
              m_options.type = BurnOptions::DataCD;
              break;
            case 1:
              m_options.type = BurnOptions::AudioCD;
              break;
          }
          
          burnDisc( );
          
          delete m_burnDiscWizard;
      
          mimeType( "text/html" );          
          
          QByteArray theData;
          
          if( !m_cdrecordFailed )
            getHTMLByteArray( Success, theData );
          
//            theData = QString( "<html><head><title>" ).append( i18n("Burn Disc") ).append( "</title></head><body><h1>" ).append( i18n("Burning completed successfully") ).append( "</h1><p>" ).append( i18n("Please click <a href=\"burn:/\">here</a> to go back, or <a href=\"burn:/Log\">here</a> for a full log.")).append( "</p></body></html>" ).local8Bit();
          else
            getHTMLByteArray( Failure, theData );
            //theData = QString( "<html><head><title>" ).append( i18n("Burn Disc") ).append( "</title></head><body><h1>" ).append( i18n("Burning failed") ).append( "</h1><p>" ).append( i18n("Please click <a href=\"burn:/Log\">here</a> for a full log of what went wrong.")).append( "</p></body></html>" ).local8Bit();
             
          data( theData );
          data(QByteArray());
          finished();
          
          return;                
        
      }
      
      delete m_burnDiscWizard;
      
      error( KIO::ERR_SLAVE_DEFINED, i18n( "User cancelled operation" ) );
      
    }  
    else if( url.path().find( QString( "/" ).append( i18n( "Settings" ) ) ) > -1 ) 
    {
      
      KWizard *settingsWizard = new SettingsWizard(m_config);
      
      connect( settingsWizard->cancelButton(), SIGNAL( clicked() ),
               this, SLOT( exitQAppLoop() ) );
      connect( settingsWizard->finishButton(), SIGNAL( clicked() ),
               this, SLOT( exitQAppLoop() ) );

      settingsWizard->show();
      
      KWin::raiseWindow( settingsWizard->winId() ); //shouldn't really be using this function but if we don't then
                                                //konq takes our focus
      KWin::forceActiveWindow( settingsWizard->winId() );
      
      qApp->enter_loop();
      delete settingsWizard;
      //delete settingsPage;
      mimeType( "text/html" );

      QByteArray theData;
        
      if( settingsWizard->result() == QDialog::Accepted )
        getHTMLByteArray( SettingsChanged, theData );
      else
        getHTMLByteArray( SettingsNotChanged, theData );
         
      data( theData );
      data(QByteArray());
      finished();                
    }
    else if( m_data_rxx->search( url.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(url.path());
      
      if( m_data_rxx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
        path.replace( *m_data_rxx , *m_data_dir );
      else if ( m_data_rxx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
        path.replace( *m_data_rxx, *m_audio_dir );
      
      KURL newurl;
      newurl.setProtocol( "file" );
      newurl.setPath( path );  
      
      //first get the mime type
      m_mimeTypeJob = KIO::mimetype( newurl, false );
      connect( m_mimeTypeJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( mimeTypeJobFinished( KIO::Job* ) ) );
      qApp->enter_loop();
      //delete m_mimeTypeJob;
      
      //then transfer the data
      m_transferJob = KIO::get( newurl, false, false );
      connect( m_transferJob, SIGNAL( data( KIO::Job *, const QByteArray & ) ), this, SLOT( getJobData( KIO::Job *, const QByteArray & ) ) );
      connect( m_transferJob, SIGNAL( totalSize( KIO::Job *, KIO::filesize_t ) ), this, SLOT( slotTotalSize( KIO::Job *, KIO::filesize_t ) ) );
      connect( m_transferJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( simpleFinish( KIO::Job* ) ) );
      qApp->enter_loop();
      //delete m_transferJob;
      
      finished();
    }
    else if( url.path().find( QString( "/" ).append( i18n( "Log" ) ) ) > -1 ) 
    {
      mimeType( "text/plain" );
      m_cdrecordLog->open( IO_ReadOnly );
      
      QByteArray theData(m_cdrecordLog->readAll());
      
      m_cdrecordLog->close();
      data( theData );
      data( QByteArray() );
      
      finished();
      
    }
    else
    {  
        //url not recognised
        error(KIO::ERR_DOES_NOT_EXIST, "" );
    }
  }

  
void kio_burnProtocol::put( const KURL & url, int permissions, bool overwrite, bool resume )
  {

    if( m_data_rxx->search( url.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(url.path());
      
      if( m_data_rxx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
        path.replace( *m_data_rxx , *m_data_dir );
      else if ( m_data_rxx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
      {
        path.replace( *m_data_rxx, *m_audio_dir );
        permissions = 0644;//this is needed so that we are able to open the file later on
      }

      
      KURL newurl;
      newurl.setProtocol( "file" );
      newurl.setPath( path );  
            
      m_transferJob = KIO::put( newurl, permissions, overwrite, resume, false );
      
      connect( m_transferJob, SIGNAL( dataReq(KIO::Job *, QByteArray &) ), this, SLOT( putDataReq( KIO::Job *, QByteArray & ) ) );
      connect( m_transferJob, SIGNAL( result( KIO::Job * ) ),this, SLOT( simpleFinish( KIO::Job * ) ) );
      
      qApp->enter_loop();
      finished();
      
    }
    else
    {  
      error(KIO::ERR_WRITE_ACCESS_DENIED, "" );
    }  
    
  }
  
void kio_burnProtocol::del( const KURL & url, bool /*isFile*/ )
  {

    if( m_data_rxx->search( url.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(url.path());
      
      if( m_data_rxx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
        path.replace( *m_data_rxx , *m_data_dir );
      else if ( m_data_rxx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
        path.replace( *m_data_rxx, *m_audio_dir );
      
      KURL newurl;
      newurl.setProtocol( "file" );
      newurl.setPath( path );
      
      m_deleteJob = KIO::del( newurl ); 
      
      connect( m_deleteJob, SIGNAL( result(KIO::Job *) ), this, SLOT( simpleFinish(KIO::Job *) ) );
      qApp->enter_loop();
      finished();
    }
    else
    {  
      //url not recognised
      error(KIO::ERR_CANNOT_DELETE, "" );
    }
  }

void kio_burnProtocol::rename( const KURL& src, const KURL& dest, bool overwrite)
  {

    KURL *newsrc = new KURL, *newdest = new KURL;
    
    if( m_data_rxx->search( src.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(src.path());
      
      if( m_data_rxx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
      {
        path.replace( *m_data_rxx , *m_data_dir );
        newsrc = new KURL();
        newsrc->setProtocol( "file" );
        newsrc->setPath( path );
      
      }
      else if ( m_data_rxx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
      {
        path.replace( *m_data_rxx, *m_audio_dir );
        newsrc = new KURL();
        newsrc->setProtocol( "file" );
        newsrc->setPath( path );
      }
      
      m_data_rxx->search( dest.path() );
      path = dest.path();
      if( m_data_rxx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
      {
        path.replace( *m_data_rxx , *m_data_dir );
        newdest = new KURL();
        newdest->setProtocol( "file" );
        newdest->setPath( path );
      
      }
      else if ( m_data_rxx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
      {
        path.replace( *m_data_rxx, *m_audio_dir );
        newdest = new KURL();
        newdest->setProtocol( "file" );
        newdest->setPath( path );
      }      
          
      m_simpleJob = KIO::rename( *newsrc, *newdest, overwrite );
      connect( m_simpleJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( simpleFinish( KIO::Job* ) ) );
      qApp->enter_loop();
      finished();
    }
    else
    {  
      error(KIO::ERR_ACCESS_DENIED, "" );
      finished();
    }
  }
  
void kio_burnProtocol::symlink( const QString& target, const KURL& dest, bool overwrite)
  {

    if( m_data_rx->search( dest.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(dest.path());
      
      if( m_data_rx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
        path.replace( *m_data_rx , *m_data_dir );
      else if ( m_data_rx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
      {
        error(KIO::ERR_CANNOT_SYMLINK, "" ); //can't have symlinks on an audio cd
        return;
      }
      
      KURL newdest;
      newdest.setProtocol( "file" );
      newdest.setPath( path );
      
      m_simpleJob = KIO::symlink( target, newdest, overwrite, false );
      connect( m_simpleJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( simpleFinish( KIO::Job* ) ) );
      qApp->enter_loop();
      finished();      
    }
    else
    {  
      error(KIO::ERR_CANNOT_SYMLINK, "" );
    }
  }
  
void kio_burnProtocol::chmod( const KURL &url, int permissions )
  {
    //i guess its a little silly to allow chmodding since we're going to force everything to read only later...
    if( m_data_rxx->search( url.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(url.path());
      
      if( m_data_rxx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
        path.replace( *m_data_rxx , *m_data_dir );
      else if ( m_data_rxx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
        path.replace( *m_data_rxx, *m_audio_dir );
      
      KURL newurl;
      newurl.setProtocol( "file" );
      newurl.setPath( path );
      
      m_simpleJob = KIO::chmod( newurl, permissions );
      connect( m_simpleJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( simpleFinish( KIO::Job* ) ) );
      qApp->enter_loop();
      finished();  
    }
    else
    {  
      error(KIO::ERR_CANNOT_CHMOD, "" );
    }
    
  }  

void kio_burnProtocol::mkdir( const KURL &url, int permissions )
  {

    if( m_data_rxx->search( url.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(url.path());
      
      if( m_data_rxx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
        path.replace( *m_data_rxx , *m_data_dir );
      else if ( m_data_rxx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
      {  
        error(KIO::ERR_COULD_NOT_MKDIR, "" );
        return; //audio cd's don't have directories, sorry
      }
      
      KURL newurl;
      newurl.setProtocol( "file" );
      newurl.setPath( path );
            
      m_simpleJob = KIO::mkdir( newurl, permissions );
      connect( m_simpleJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( simpleFinish( KIO::Job* ) ) );
      qApp->enter_loop();
      finished();  
    }
    else
    {  
      error(KIO::ERR_COULD_NOT_MKDIR, "" );
    }
    
  }  
  
void kio_burnProtocol::rmdir( const KURL &url )
  {

    if( m_data_rxx->search( url.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(url.path());
      
      if( m_data_rxx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
        path.replace( *m_data_rxx , *m_data_dir );
      else if ( m_data_rxx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
      {  
        error(KIO::ERR_COULD_NOT_RMDIR, "" );
        return;//surely we could never get here anyway
      }
      
      KURL newurl;
      newurl.setProtocol( "file" );
      newurl.setPath( path );
      
      m_simpleJob = KIO::rmdir( newurl );
      connect( m_simpleJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( simpleFinish( KIO::Job* ) ) );
      qApp->enter_loop();
      finished();  
    }
    else
    {
      error( KIO::ERR_COULD_NOT_RMDIR, "");
    }
    
  }  
void kio_burnProtocol::listDir( const KURL& url )
  {
    kdDebug() << "kio_burnProtocol::listDir( const KURL& url )" << endl;

  //we're in the top level directory (e.g. burn:/)
  //return a list of standard files - these are all imaginary files
  

    if ( url.path() == "/" )
    {
      //we're in the top level directory
      //we need to report a "burn disc" type file, for actually burning the disc
      //and a directory called data where data to be burned to the cd is kept
      
      totalSize( 2 );
      
      UDSEntry entry;
      UDSAtom atom;
      app_file( entry, i18n("Burn Disc"), 0, 0400);
      atom.m_uds = KIO::UDS_ICON_NAME;
      atom.m_str = "cdwriter_unmount";
      entry.append( atom );
      listEntry( entry, false );
      
      app_file( entry, i18n("Settings"), 0, 0400 );
      atom.m_str = "configure";
      entry.append( atom );
      listEntry( entry, false );

      app_dir( entry, i18n("Data"), 0, 0400 );
      listEntry( entry, false );
      
      app_dir( entry, i18n("Audio CD"), 0, 0400 );
      listEntry( entry, false );

      listEntry( UDSEntry(), true );

      finished();
    }

    else if( m_data_rx->search( url.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(url.path());
      
      if( m_data_rx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
        path.replace( *m_data_rx , *m_data_dir );
      else if ( m_data_rx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
        path.replace( *m_data_rx, *m_audio_dir );
      
      KURL newurl;
      newurl.setProtocol( "file" );
      newurl.setPath( path );
      
      m_listJob = ::KIO::listDir( newurl, false, true );
      
      connect( m_listJob, SIGNAL( entries(KIO::Job *, const KIO::UDSEntryList&) ), this, SLOT( listJobEntries( KIO::Job *, const KIO::UDSEntryList&) ) );
      connect( m_listJob, SIGNAL( totalSize( KIO::Job *, KIO::filesize_t ) ), this , SLOT( slotTotalSize( KIO::Job *, KIO::filesize_t ) ) );
      connect( m_listJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( listJobFinished( KIO::Job * ) ) );
      qApp->enter_loop();
      finished();
    }          
    else
    {
 
      error(KIO::ERR_DOES_NOT_EXIST, "" );

      finished();
    }


  }

void kio_burnProtocol::stat( const KURL &url )
  {
  
    if( url.path() == "/" )
    {
      //in top level directory
      UDSEntry entry;
      app_dir( entry,"/", 0, 0400 );
      statEntry( entry ) ;
      finished();

    }

    else if( url.path().find( QString( "/" ).append( i18n( "Burn Disc" ) ) ) > -1 ) 
    {
      UDSEntry entry;
      app_file( entry, i18n("Burn Disc").append( KIO::convertSize( KDirSize::dirSize( KURL( *m_data_dir ) ) ) ), 0, 0400 );
      statEntry( entry );
      finished();
    }
    
    else if( url.path().find( QString( "/" ).append( i18n( "Settings" ) ) ) > -1 ) 
    {
      UDSEntry entry;
      app_file( entry, i18n(""), 0, 0400 );
      statEntry( entry );
      finished();
    }
    
    else if( url.path().find( QString( "/" ).append( i18n( "Log" ) ) ) > -1 ) 
    {
      UDSEntry entry;
      app_file( entry, i18n("Log"), 0, 0400 );
      statEntry( entry );
      finished();
    }

    else if( m_data_rx->search( url.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(url.path());
      
      if( m_data_rx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
        path.replace( *m_data_rx , *m_data_dir );
      else if ( m_data_rx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
        path.replace( *m_data_rx, *m_audio_dir );
      
      KURL newurl;
      newurl.setProtocol( "file" );
      newurl.setPath( path );
      
      m_statJob = ::KIO::stat( newurl, false ); 
      connect( m_statJob, SIGNAL( result( KIO::Job* ) ), this, SLOT( statJobFinished( KIO::Job*  ) ) );
      qApp->enter_loop();
      finished();
    }
    else
    {
      
      //don't recognise this url, pretend its a file, and when it is get()'d we'll return an error
      
      error(KIO::ERR_DOES_NOT_EXIST, "" );
      finished();
    }


  }

  
void kio_burnProtocol::mimetype(const KURL & url)
  {
  

    if( m_data_rx->search( url.path() ) > -1 )
    {
      //deal with data directories ("Data", "Audio CD", in theory more could be added)
      QString path(url.path());
      
      if( m_data_rx->capturedTexts()[0].find( i18n("Data" ) ) > -1 )
        path.replace( *m_data_rx , *m_data_dir );
      else if ( m_data_rx->capturedTexts()[0].find( i18n("Audio CD" ) ) > -1 )
        path.replace( *m_data_rx, *m_audio_dir );
      
      KURL newurl;
      newurl.setProtocol( "file" );
      newurl.setPath( path );
      
    //first get the mime type
      m_mimeTypeJob = KIO::mimetype( newurl, false );
      connect( m_mimeTypeJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( mimeTypeJobFinished( KIO::Job* ) ) );
      qApp->enter_loop();
      delete m_mimeTypeJob;
      finished();
    } 
    
    else if( url.path().find( QRegExp(QString( "^/").append( i18n("Burn Disc") ) ) ) > -1 )
    {
      mimeType( "kdedevice/cdwriter_unmount" );
      finished();
    }
    
    else{
  //don't know - have a guess
      mimeType("text/plain");
      finished();
    }
  
  }

  /*********************SLOTS***************/  
void kio_burnProtocol::slotProcessExited( KProcess *process )
{
    
  if( !process->normalExit() )
  {   
    if( QString( process->name() ).find(  "mkisofsproc" ) > -1 )
      error( KIO::ERR_SLAVE_DEFINED,
             i18n( "mkisofs failed with exit status: %1" ).arg(  process->exitStatus() ) ); 
    else
      error( KIO::ERR_SLAVE_DEFINED,
             i18n( "cdrecord failed with exit status: %1 " ).arg( process->exitStatus()  ) ); 
  }
      
  if( QString( process->name() ).find(  "cdrecordproc" ) > -1 ) 
    //note that "cdrecordproc" refers to growisofs as well - 
    //basically this conditional says "if the disc has finished burning"
  {
//  if( m_cdrecordFailed )
//  {
//    //error( KIO::ERR_SLAVE_DEFINED,
//    //       i18n( "cdrecord failed" ) );
//    //this is now dealt with elsewhere - instead of returning an error the burnDisc() function
//   //picks up that an error has occured, and the user will be given the chance to view the cdrecord
//    //log
//  }
    
    setProgressBarShown( false );
      
  }
    
  qApp->exit_loop();
}
  
void kio_burnProtocol::mkisofsStdout( KProcess */*process*/, char * buffer, int buflen )
{
  /*mkisofs has produced some output, if it looks like a progress number, use it to change the progress bar
  in the progress dialog*/
  static QRegExp rx( " ([0-9]{1,2})\\." );
  
  QString s = QString::fromLatin1( buffer, buflen );
  if( rx.search( s ) >-1 ){
    QStringList list = rx.capturedTexts();
    if( m_notDcop )
    {
      m_progressDialog->progressBar->setProgress( list[1].toInt() );
    }else{
      QByteArray data;
      QDataStream arg( data, IO_WriteOnly );
      arg << list[1].toInt();
      KApplication::dcopClient()->send( QCString( m_dcopId ) , 
                                        QCString( "kio_burnPlugin" ), 
                                        QCString("setProgressBarValue( int )"), 
                                        data);
    }
  }
}
  
void kio_burnProtocol::growisofsStdout( KProcess */*process*/, char * buffer, int buflen )
{
    
  m_cdrecordLog->writeBlock( buffer, buflen );
    
  QString s = QString::fromLatin1( buffer, buflen );
  QStringList sl = QStringList::split( "\n", s );
    
  static QRegExp rx( "(\\d\\d)\\..*%" ); 

  QStringList::iterator it;
  
  for( it = sl.begin() ; it != sl.end(); it++ )
  {
    if( rx.search( *it ) > -1 )
    {
      int progress = (rx.capturedTexts())[1].toInt();
      
      if( m_notDcop )
      {
        m_progressDialog->progressBar->setProgress( progress );
      }else{
        QByteArray data;
        QDataStream arg( data, IO_WriteOnly );
        arg << progress;
        KApplication::dcopClient()->send( QCString( m_dcopId ) , 
        QCString( "kio_burnPlugin" ), 
        QCString("setProgressBarValue( int )"), 
        data);
        
      }

    }
  }
  if ( s.contains( "writing lead-out" ) )
  {
    //looks like a success
    m_cdrecordFailed = false; 
  }
}
  
void kio_burnProtocol::cdrecordStdout( KProcess */*process*/, char * buffer, int buflen )
  {
    /*cdrecord has given some output*/
    
    //before anything else, log it
    m_cdrecordLog->writeBlock( buffer, buflen );
    
    //then use the data to create a progress in percentage
    QString s = QString::fromLatin1( buffer, buflen );
    static QRegExp rx( "Total size: *([0-9]+) MB" );
    static QRegExp rx_2( "Track \\d\\d: *([1-9][0-9]*) of *([1-9][0-9]*)" );
    int progress, totalSize;
    static int previousTracks; //will store the size of the tracks that have already been written
    
    if( rx_2.search( s ) >-1 ) //recieved some progress information
    {
      QStringList list = rx_2.capturedTexts();
      progress = list[1].toInt() + previousTracks;
      
      if (list[1].toInt() == list[2].toInt()) previousTracks = progress;
      
      if( m_notDcop )
      {
        m_progressDialog->progressBar->setProgress( progress );
      }else{
        QByteArray data;
        QDataStream arg( data, IO_WriteOnly );
        arg << progress;
        KApplication::dcopClient()->send( QCString( m_dcopId ) , 
                                          QCString( "kio_burnPlugin" ), 
                                          QCString("setProgressBarValue( int )"), 
                                          data);
        
      }
    }
    
    else if( rx.search( s ) >-1 )//do the total data size when we get it
    {
      //getting here implies we've started a new write process, so no previous tracks have been written
      previousTracks = 0;
      
      //also, being here is the best indication we can get that 
      
      QStringList list = rx.capturedTexts();
      totalSize = list[1].toInt();
      if( totalSize )
        if( m_notDcop )
        {
          m_progressDialog->progressBar->setTotalSteps( totalSize );
        }else{
          QByteArray data;
          QDataStream arg( data, IO_WriteOnly );
          arg << totalSize;
          KApplication::dcopClient()->send( QCString( m_dcopId ) , 
                                            QCString( "kio_burnPlugin" ), 
                                            QCString("setProgressBarTotal( int )"), 
                                            data);
        }
      else
      {
        //writing less that 1MB, just set the progres bar to 99% and leave it
        if( m_notDcop )
        {
          m_progressDialog->progressBar->setTotalSteps( 100 );
          m_progressDialog->progressBar->setProgress( 99 );
        }else{
          QByteArray *data = new QByteArray;
          QDataStream *arg = new QDataStream( *data, IO_WriteOnly );
          *arg << 100;
          KApplication::dcopClient()->send( QCString( m_dcopId ) , 
                                            QCString( "kio_burnPlugin" ), 
                                            QCString("setProgressBarTotal( int )"), 
                                            *data);
          delete data, arg;
          data = new QByteArray;
          arg = new QDataStream( *data, IO_WriteOnly );
          *arg << 99;
          KApplication::dcopClient()->send( QCString( m_dcopId ) , 
                                            QCString( "kio_burnPlugin" ), 
                                            QCString("setProgressBarValue( int )"), 
                                            *data);
        }
      }
    }
        
    if( s.contains( "Fixating..." ) )
    {
      //looks like we've got to the final stage of the process
      setProgressBarTotal(100);
      setProgressBarProgress(99);
      setProgressBarText( i18n( "Fixating" ) );
               
    }
    if( s.find( "Average write speed" ) > -1 )
    {
      //looks like cdrecord finished successfully
      m_cdrecordFailed = false;
    }
    
  }

void kio_burnProtocol::slotTotalSize(  KIO::Job *, KIO::filesize_t size) 
  {
    /*We've received a totalSize from some slave, pass it on to our controlling job*/  
    totalSize( size ); 
  }
  
void kio_burnProtocol::mimeTypeJobFinished( KIO::Job *job )
  {
  
    uint i;

  //has there been an error?
    if( (i = job->error()) ){
      error( i, "" );
    }
    
    this->mimeType( m_mimeTypeJob->mimetype() );
    qApp->exit_loop();
    
      
  }

      
void kio_burnProtocol::getJobData( KIO::Job */*job*/, const QByteArray &data )
  {   
    this->data( data ); //just pass it on
  }

void kio_burnProtocol::simpleFinish( KIO::Job *job )
  { 
    
    uint i;

  //has there been an error?
    if( (i = job->error()) ){
      error( i, "" );
    }
    
    qApp->exit_loop();    
    
  }


void kio_burnProtocol::listJobEntries(KIO::Job */*job*/, const KIO::UDSEntryList &list )
  {

    if( list.isEmpty() )
      return;

    for (uint z=0; z < list.count(); z++)
    {
      KFileItem *item = new KFileItem (list[z], "");

      if ((item->name() != QString (".")) && (item->name() != QString ("..")))
          listEntry (list[z], false);
    }

  }

void kio_burnProtocol::listJobFinished( KIO::Job * job ){
  int i;

  //has there been an error?
  if( (i = job->error()) ){
    error( i, "" );
  }

  listEntry( UDSEntry(), true );
  qApp->exit_loop();
  //delete job;
}
  
  
void kio_burnProtocol::statJobFinished( KIO::Job * job )
  {
    int i;
    //has there been an error?
    if( (i = job->error()) ){
      error( i, "" );
    }
    //a stat job on the data directory is complete
    //we copy it to the job controlling us
    statEntry( m_statJob->statResult() );
    qApp->exit_loop();
  }
  
  
void kio_burnProtocol::putDataReq( KIO::Job *, QByteArray &data )
  {
  
    dataReq();
    
    readData( data );  
  }

  void kio_burnProtocol::exitQAppLoop()
  {

    qApp->exit_loop();

  }
extern "C"
  {
    int kdemain(int argc, char **argv)
    {
      //normally a slave would be a KInstance, but since we're going to be calling other slaves (namely file:/)
      //we have to be a KApplication
      KApplication app( argc, argv, "kio_burn", false, true );

      kdDebug(7101) << "*** Starting kio_burn " << endl;

      if (argc != 4)
        {
          kdDebug(7101) << "Usage: kio_burn  protocol domain-socket1 domain-socket2" << endl;
          exit(-1);
        }

      kio_burnProtocol slave(argv[2], argv[3]);
      slave.dispatchLoop();

      kdDebug(7101) << "*** kio_burn Done" << endl;
      return 0;
    }
  }


/*!
    \fn kio_burnProtocol::getHTMLByteArray( const QString  &file )
            Turn html result template into byte array suitable for sending back to client
 */
void kio_burnProtocol::getHTMLByteArray( ResultTemplate t, QByteArray &dest ) const
{
  if( t == Success )
  {
    QFile templateFile( locate( "data", "kio_burn/templates/success.html" ) );
    
    if(!templateFile.open( IO_ReadOnly ) )
    {
      dest = QCString( "Finished burning successfully");
      return;
    }
    
    char *buffer = new char[3001];
    
    templateFile.readBlock( buffer, 3000 );
    QString s = QString::fromUtf8( buffer );

    s = s.replace( "%1", i18n( "Burning finished successfully" ))
        .replace( "%2", "file:" + locate( "icon" , "default.kde/32x32/devices/cdrom_unmount.png" ) )
        .replace( "%3", i18n( "Open CD/DVD" ) )
        .replace( "%4", "file:" + locate( "icon" , "default.kde/32x32/devices/cdwriter_unmount.png" ) )
        .replace( "%5", i18n( "Burn this disc again" ) )
        .replace( "%6", "file:" + locate( "icon" , "default.kde/32x32/actions/edit.png" ) )
        .replace( "%7", i18n( "Create a new disc" ) )
        .replace( "%8", "file:" + locate( "icon" , "default.kde/32x32/actions/toggle_log.png" ) )
        .replace( "%9", i18n( "View log" ) );
    
    
    dest = s.utf8();
    
    templateFile.close();
    
    delete buffer;
  }
  if( t == Failure )
  {
    QFile templateFile( locate( "data", "kio_burn/templates/failure.html" ) );
    
    if(!templateFile.open( IO_ReadOnly ) )
    {
      dest = QCString( "Could not burn disc");
      return;
    }
    
    char *buffer = new char[3001];
    
    templateFile.readBlock( buffer, 3000 );
    QString s = QString::fromUtf8( buffer );

    s = s.replace( "%1", i18n( "Burning failed" ))
        .replace( "%2", "file:" + locate( "icon" , "default.kde/32x32/actions/edit.png" ) )
        .replace( "%3", i18n( "View log" ) )
        .replace( "%4", "file:" + locate( "icon" , "default.kde/32x32/devices/cdwriter_unmount.png" ) )
        .replace( "%5", i18n( "Go back to burn:/" ) );
    
    
    dest = s.utf8();
    
    templateFile.close();
    
    delete buffer; 
  }
  if( t == SettingsChanged )
  {
    QFile templateFile( locate( "data", "kio_burn/templates/settingschanged.html" ) );
    
    if(!templateFile.open( IO_ReadOnly ) )
    {
      dest = QCString( "New settings saved");
      return;
    }
    
    char *buffer = new char[3001];
    
    templateFile.readBlock( buffer, 3000 );
    QString s = QString::fromUtf8( buffer );

    s = s.replace( "%1", i18n( "New settings saved" ))
        .replace( "%2", "file:" + locate( "icon" , "default.kde/32x32/devices/cdwriter_unmount.png" ) )
        .replace( "%3", i18n( "Go back to burn:/" ) );
    
    
    dest = s.utf8();
    
    templateFile.close();
    
    delete buffer; 
  }  
  if( t == SettingsNotChanged )
  {
    QFile templateFile( locate( "data", "kio_burn/templates/settingschanged.html" ) );
    
    if(!templateFile.open( IO_ReadOnly ) )
    {
      dest = QCString( "Settings not changed");
      return;
    }
    
    char *buffer = new char[3001];
    
    templateFile.readBlock( buffer, 3000 );
    QString s = QString::fromUtf8( buffer );

    s = s.replace( "%1", i18n( "Settings not changed" ))
        .replace( "%2", "file:" + locate( "icon" , "default.kde/32x32/devices/cdwriter_unmount.png" ) )
        .replace( "%3", i18n( "Go back to burn:/" ) );
    
    
    dest = s.utf8();
    
    templateFile.close();
    
    delete buffer; 
  } 
}
