/***************************************************************************
*   Copyright (C) 2004 by James Thorniley                                 *
*   james@uncommonlygood.co.uk                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "burn.h"
#include "progressdlg.h"
#include "burndiscwizard.h"
#include "playlistpage.h"

#include <kapplication.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <dcopclient.h>
#include <kdebug.h>
#include <kdirlister.h>
#include <klistview.h>
#include <kconfig.h>
#include <qlabel.h>
#include <kprocess.h>

#include <qfile.h>
#include <qregexp.h>
#include <qdir.h>
#include <qstringlist.h>
#include <qtimer.h>

#include <akode/localfile.h>
#include <akode/audiobuffer.h>
#include <akode/frametostream_decoder.h>
#include <akode/streamtoframe_decoder.h>

#include <unistd.h>
#include <errno.h>

typedef struct{
  QString name;
  QString type;
} AudioFile;

void kio_burnProtocol::burnDisc( )
{

  if( m_config->readBoolEntry( "lock" ) )
  {
    //looks like we're already writing a cd
    int result = KMessageBox::warningContinueCancel( 0,
                                        i18n( "It looks like a disc is already being burnt. It is recommended that you click cancel and wait for the existing burning process to finish (watch the light on your CD burner). If you are sure that nothing is already being burnt, you may click continue."),
                                        i18n( "Burning in progress!" )
                                      );
    
    if( result == KMessageBox::Cancel )
    {
      return;
    }
                                            
  }
  
  m_config->writeEntry( "lock", true );
  m_config->sync();
  
  //will be needed for audio CDs
  QValueList<AudioFile> audioFiles;
  QListViewItem *item;
  QStringList nameList;
  AudioFile a;
  
  for ( item = m_burnDiscWizard->playlistPage()->trackListView->firstChild(); item; item = item->itemBelow() )
  {

    a.name = item->text(4);
    a.type = item->text(3);
      
    audioFiles.append( a );
    nameList.append( a.name );
  }

  m_config->writeEntry( "tracklist", nameList );
  m_config->sync();

  if( m_dcopId == QString::null ) //if m_dcopId is null it means we were not called from the konqueror plugin
                                  //(hopefully)
  {
    //create a progress dialog for mkisofs
    m_progressDialog = new ProgressDlg( 0 , "burnprogressdlg" );
    m_notDcop = true;
  }
  else
  {
    m_progressDialog = 0;
    m_notDcop = false;
  }
  
  setProgressBarText( i18n( "Preparing to burn disc" ) );
  setProgressBarShown( true );
  
  if ( m_options.type == BurnOptions::AudioCD )
  {
  
    m_burnStage = decodeaudio;
    //need to convert ogg and mp3 to wav.
    
    
    QValueList<AudioFile>::Iterator file;
    int count = 0;
    
    //count the number of files that need to be decoded
    for( file = audioFiles.begin() ; file != audioFiles.end(); ++file )
    {
      if( (*file).type == i18n("Ogg Vorbis audio") 
            || (*file).type == i18n("MP3 audio" )
            /*|| (*file).type == i18n("Ogg FLAC audio" )*/)
      {
        count++;
      }
    }

    setProgressBarTotal( count );
          
    int progress = 1;
    
    for( file = audioFiles.begin() ; file != audioFiles.end(); ++file  )
    {
     
      if((*file).type == "Wav audio" )
      {
        continue; //no decoding necessary
      }
      
      //open a wav file to output to
      QString outputFileName((*file).name);
      outputFileName.append(".wav" ) ;
                
      //create configuration for new audiofile
      AFfilesetup afsetup = afNewFileSetup();
      afInitFileFormat( afsetup, AF_FILE_WAVE );
        
      //now open the file
      m_afhandle = afOpenFile( (locateLocal( "data", "kio_burn/decode/" ) += outputFileName).local8Bit(),
                                "w",
                                afsetup );

      if( m_afhandle == AF_NULL_FILEHANDLE )
      {
        //uh-oh, should probably report some kind of error
        kdDebug() << "Decoding audio failed, could not open output file: " <<  outputFileName << endl;
        afFreeFileSetup( afsetup );
        continue;
      }
      

      const char *plugin;
      if((*file).type == i18n("Ogg Vorbis audio")
           /*|| (*file).type == i18n("Ogg FLAC audio")*/)      
        plugin =  "xiph";
      else if((*file).type == i18n("MP3 audio"))
        plugin = "mpeg";
      else
        plugin = "none"; //won't (shouldn't) happen but avoids compiler warnings
      
      aKode::DecoderPluginHandler *decoderPlugin = new aKode::DecoderPluginHandler( plugin );
      
      if (!decoderPlugin->isLoaded()) {
        kdDebug() << "Could not load decoder-plugin: " <<  plugin << endl;
        continue;
      }
            
      //akode only works with relative path names, so we'll have to find out our cwd

      char *wd = cwd();
            
      if( wd == NULL )
        continue;
      
      QString *relativeFileName = new QString();
      
      for( int subdirs = QString( wd ).find("/"); subdirs > 0; subdirs-- )
      {
        relativeFileName->append( "../" ); 
      } 
      
      delete wd;
      
      (*relativeFileName) += locateLocal( "data", "kio_burn/audio/" );
      (*relativeFileName) += (*file).name;
      
      aKode::LocalFile *file = new aKode::LocalFile( *relativeFileName );
      if (!file->openRO()) {
        kdDebug() << "Unreadable file: " << *relativeFileName << endl;
      } else
        file->close();
       
       
      aKode::FrameDecoder *decoder = decoderPlugin->openFrameDecoder(file);
     
      aKode::AudioBuffer *buffer = new aKode::AudioBuffer(16);
      aKode::StreamDecoder *sdecoder;
      sdecoder = new aKode::FrameToStreamDecoder(decoder);
      m_decoder = new aKode::StreamToFrameDecoder(sdecoder, buffer);
      
      QTimer timer( this );
      connect( &timer, SIGNAL( timeout() ), this, SLOT( decodeToWav() ) );
      connect( this, SIGNAL( decoderFinished() ), this, SLOT( exitQAppLoop() ) ); 
      
      timer.start(0);
               
      qApp->enter_loop();

      //now close all the files
      afCloseFile( m_afhandle );
      afFreeFileSetup( afsetup );
        
      setProgressBarProgress( progress );
      
      progress++;
      
      delete buffer;
      delete decoder;
      delete m_decoder;
      delete sdecoder;
      delete decoderPlugin;
      delete relativeFileName;
      delete file;
      
        
    }
  }
  
  //mkisofs stuff is only necessary for data CD
  if ( (m_options.type == BurnOptions::DataCD) )
  {

    m_burnStage = mkisofs;
    
    setProgressBarTotal( 100 );
    
    m_process = new KProcess( this, "mkisofsproc" );

    m_process->setWorkingDirectory( locateLocal( "data", "kio_burn/" ) );
    *m_process << "mkisofs";
    *m_process << "-r"
    << "-gui"
    << "-o" << locateLocal( "data", "kio_burn/discimage.iso" )
    << locateLocal( "data", "kio_burn/data/" ) ;


    connect( m_process, SIGNAL( processExited( KProcess * ) ),
             this, SLOT( slotProcessExited( KProcess * ) ) );
    connect( m_process, SIGNAL( receivedStderr( KProcess *, char*, int ) ),
             this, SLOT( mkisofsStdout( KProcess*, char*, int ) ) );

    if ( m_process->start( KProcess::NotifyOnExit, KProcess::Stderr ) )
    {
      qApp->enter_loop();
    }
    else
    {
      error( KIO::ERR_SLAVE_DEFINED,
             QString( "Could not run mkisofs" ) );
      m_config->writeEntry( "lock", false );
      m_config->sync();
      return ;
    }

    if ( !m_process->normalExit() )
    {
      //mkisofs failed, we should give up the operation here
      //an error message should already have been sent

      QFile::remove
        ( locateLocal( "data", "kio_burn/discimage.iso" ) );
      delete m_process;
      m_config->writeEntry( "lock", false );
      m_config->sync();
      return ;
    }

    disconnect( m_process, SIGNAL( receivedStderr( KProcess *, char*, int ) ),
                this, SLOT( mkisofsStdout( KProcess*, char*, int ) ) );
    
    delete m_process;
  }



  //ok, we've made an iso image if necessary, now we need to run cdrecord
  setProgressBarText( i18n( "Burning disc" ) );
  setProgressBarProgress( 0 );
  
  m_burnStage = cdrecord;
  
  m_process = new KProcess( this, "cdrecordproc" );
  
  //this variable will be set to false when cdrecord appears to have completed successfully
  //see kio_burnProtocol::cdrecordStdout() in burn.cpp
  m_cdrecordFailed = true;

  m_process->setWorkingDirectory( locateLocal( "data", "kio_burn/" ) );

  if( !(m_burnDiscWizard->mediaIsDVD()) )
  {
    kdDebug() << "Burning CD" ;
    *m_process << "cdrecord" ;
    *m_process << "-eject"
    << "gracetime=0"
    << "-v"
        //<< "-dummy"
        << ("--dev=" + m_config->readEntry( "file" ));
  
    if ( m_options.blankCDRW )
      * m_process << "blank=fast";
  
    if ( m_options.advSettings )
    {
      if ( m_options.doNotFixate )
          * m_process << "-nofix";
      if ( m_options.mode == BurnOptions::SAO )
        * m_process << "-dao"; //TAO is the default mode, so we don't have to specify an option for that
  
      if( m_options.burnSpeed.length() > 0 )
        *m_process << ( QString( "speed=" ) += m_options.burnSpeed );
    }
  
    switch ( m_options.type )
    {
      case BurnOptions::DataCD:
        *m_process << "discimage.iso";
        break;
         
      case BurnOptions::AudioCD:
        *m_process << "-pad" ;
        QValueList<AudioFile>::Iterator file;
        for ( file = audioFiles.begin(); file != audioFiles.end(); ++file )
        {
          QString fileName(locateLocal( "data", "kio_burn/" ));
          
          if( ((*file).type == i18n( "Ogg Vorbis audio" )) || ((*file).type == i18n( "MP3 audio" )) )
          {
            fileName += "decode/";
            fileName += (*file).name;
            fileName += ".wav";
          }
          else
          {
            fileName += "audio/"; 
            fileName += (*file).name;
          }  

          *m_process << fileName;
          
          
        }
        break;
    }
      
   
    connect( m_process, SIGNAL( receivedStdout( KProcess *, char*, int ) ), this, SLOT( cdrecordStdout( KProcess*, char*, int ) ) );
    connect( m_process, SIGNAL( processExited( KProcess * ) ), this, SLOT( slotProcessExited( KProcess * ) ) );
  


    m_cdrecordLog->open( IO_WriteOnly | IO_Truncate );
    
    if ( m_process->start( KProcess::NotifyOnExit, KProcess::Stdout ) )
    {
      qApp->enter_loop();
    }
    else
    {
      setProgressBarShown( false );
      delete m_progressDialog;
      
      QFile::remove
          ( locateLocal( "data", "kio_burn/discimage.iso" ) );
      QDir *qdir = new QDir( locateLocal( "data", "kio_burn/decode/" ) );
      for( uint i =0; i < qdir->count(); i++ )
        QFile::remove(locateLocal( "data", "kio_burn/decode/" )+(*qdir)[i]);
      delete qdir;
 
      error( KIO::ERR_SLAVE_DEFINED,
            i18n( "Could not run cdrecord" ) );
    }
  
  
    m_cdrecordLog->close();
    
  }
    
  else
  { //burning a DVD with growisofs

    kdDebug() << "Burning DVD" << endl;
    *m_process << "/usr/bin/growisofs" ;
    //if( !(m_options.advSettings && m_options.doNotFixate) )
    //  *m_process << "-dvd-compat";
    *m_process << "-Z";
    
    QString string(m_config->readEntry( "file" ));
    string += "=discimage.iso";
    *m_process << string;
    
    // need to work out some way to set DVD recording speed
    //if( m_options.advSettings && m_options.burnSpeed.length() > 0 )
    //  *m_process << ( QString( "speed=" ) += m_options.burnSpeed );
    //    << "--dev=" + m_config->readEntry( "file" );
    //sao/tao doesn't apply, nor does fixating, really
    
    connect( m_process, SIGNAL( receivedStdout( KProcess *, char*, int ) ), this, SLOT( growisofsStdout( KProcess*, char*, int ) ) );
    connect( m_process, SIGNAL( receivedStderr( KProcess *, char*, int ) ), this, SLOT( growisofsStdout( KProcess*, char*, int ) ) );
    connect( m_process, SIGNAL( processExited( KProcess * ) ), this, SLOT( slotProcessExited( KProcess * ) ) );
  
    m_cdrecordLog->open( IO_WriteOnly | IO_Truncate );
    
    if( m_options.blankCDRW )
      m_process->setUsePty( KProcess::Stdout, false ); //if pty is not used growisofs will bail if
    //disc already has isofs
    
    if ( m_process->start( KProcess::NotifyOnExit, (KProcess::Communication)(KProcess::Stdout | KProcess::Stderr) ) )
    {
      qApp->enter_loop();
    }
    else
    {
      setProgressBarShown( false );
      delete m_progressDialog;
      
      QFile::remove
          ( locateLocal( "data", "kio_burn/discimage.iso" ) );
      QDir *d = new QDir( locateLocal( "data", "kio_burn/decode/" ) );
      for( uint i =0; i < d->count(); i++ )
        QFile::remove(locateLocal( "data", "kio_burn/decode/" ) + (*d)[i] );
      delete d;
       
      error( KIO::ERR_SLAVE_DEFINED,
             i18n( "Could not run growisofs" ) );
    }
  
  
    m_cdrecordLog->close();
  }
  m_cdrecordLog->open( IO_ReadOnly );
    //ok, everything was successful
  
  if ( m_notDcop )
  {
    if ( !m_cdrecordFailed )
      KMessageBox::information( 0,
                              i18n( "Burn process complete" ),
                              i18n( "Burn Disc" ) );
    else
    {
      QString details( m_cdrecordLog->readAll() );
      KMessageBox::detailedError( 0,
                                i18n( "It looks like there was a problem burning your disc." ),
                                details,
                                i18n( "Burning failed" )); 
    }
  }

  //cleanup
  m_cdrecordLog->close();
  
  QFile::remove
    ( locateLocal( "data", "kio_burn/discimage.iso" ) );
  QDir *qdir = new QDir( locateLocal( "data", "kio_burn/decode/" ) );
  for( uint i =0; i < qdir->count(); i++ )
    QFile::remove(locateLocal( "data", "kio_burn/decode/" ) + (*qdir)[i] );
  delete qdir;
 
  delete m_process;

  m_config->writeEntry( "lock", false );
  m_config->sync();
  
}

void kio_burnProtocol::decodeToWav()
{
 
  aKode::AudioFrame *frame = new aKode::AudioFrame;

 
  if( !m_decoder->eof() && !m_decoder->error() )
  {

    m_decoder->readFrame( frame );

    int16_t **data = (int16_t**)frame->data;
    int16_t *pcmout = new int16_t[frame->length*frame->channels];
  
    for( int j = 0; j < frame->channels ; j++)
      for( int i = 0; i < frame->length; i++ )
        pcmout[frame->channels*i+j] = data[j][i];
    
    afWriteFrames( m_afhandle, AF_DEFAULT_TRACK, pcmout, frame->length );
    
    delete frame;
    delete pcmout;
    
  }
  else
  {//finished
    emit decoderFinished();
  }
  
}

char *kio_burnProtocol::cwd()
{
  int buflen = 256;
  char *cwd = new char[buflen];
  errno = 0;
  while( (getcwd(cwd,buflen) == NULL) )
  {
    if( (buflen += 10) > 4096 )
    {
      kdDebug() << "Buffer getting ridiculously large getting cwd, skipping file to avoid infinite  loop" << endl;
      return NULL;
    }
    if( errno != ERANGE )
    {
      kdDebug() << "Could not get cwd while decoding, bailing" << endl;
      return NULL;
    }
        
    delete cwd;
    cwd = new char[buflen];
        
  }
  
  return cwd;
}

void kio_burnProtocol::setProgressBarText( QString text )
{
  
  if ( m_dcopId != QString::null )  //if we have been called from the toolbar button in konqueror, we will have
    //recieved a dcop id to send progress messages to
  {
    
    QByteArray *data = new QByteArray;
    QDataStream *arg = new QDataStream( *data, IO_WriteOnly );

    *arg << text; 
    
    KApplication::dcopClient()->send( QCString( m_dcopId ),
    QCString( "kio_burnPlugin" ),
    QCString( "setProgressBarText( QString )" ),
    *data );
    delete arg;
    delete data;
  }else
  {
    m_progressDialog->textLabel->setText( text );
  }   
}

void kio_burnProtocol::setProgressBarShown( bool show )
{
  
  if ( m_dcopId != QString::null )  //if we have been called from the toolbar button in konqueror, we will have
    //recieved a dcop id to send progress messages to
  {

    QByteArray *data = new QByteArray;
    QDataStream *arg = new QDataStream( *data, IO_WriteOnly );

    data = new QByteArray;
    arg = new QDataStream( *data, IO_WriteOnly );
    *arg << show;
    KApplication::dcopClient()->send( QCString( m_dcopId ),
    QCString( "kio_burnPlugin" ),
    QCString( "showProgressBar( bool )" ),
    *data );    
    delete arg;
    delete data;

  }else
  {
    m_progressDialog->setShown( show );
  }  
}

void kio_burnProtocol::setProgressBarTotal( int total )
{
  
     //set progress bar total to total
  if( m_notDcop )
  {
    m_progressDialog->progressBar->setTotalSteps( total );
  }else
  {
  
    QByteArray *data = new QByteArray;
    QDataStream *arg = new QDataStream( *data, IO_WriteOnly );

    data = new QByteArray;
    arg = new QDataStream( *data, IO_WriteOnly );
    *arg << total;
    KApplication::dcopClient()->send( QCString( m_dcopId ) , 
    QCString( "kio_burnPlugin" ), 
    QCString("setProgressBarTotal( int )"), 
    *data);
    
    delete arg;
    delete data;
 
  }
  
 
}

void kio_burnProtocol::setProgressBarProgress( int progress )
{
         
  if( m_notDcop )
  {
    m_progressDialog->progressBar->setValue( progress );
  }else
  {
    
    QByteArray *data = new QByteArray;
    QDataStream *arg = new QDataStream( *data, IO_WriteOnly );

    data = new QByteArray;
    arg = new QDataStream( *data, IO_WriteOnly );
    *arg << progress;
    KApplication::dcopClient()->send( QCString( m_dcopId ) , 
    QCString( "kio_burnPlugin" ), 
    QCString( "setProgressBarValue( int )" ), 
    *data);
    
    delete arg;
    delete data;
  }
       
}
