/***************************************************************************
 *   Copyright (C) 2005 by James Thorniley                                 *
 *   james@uncommonlygood.co.uk                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "burndiscwizard.h"

#include "burnoptionspage.h"
#include "advancedsettingsdialog.h"
#include "playlistpage.h"
#include "driveselectpage.h"
#include "scanmediapage.h"
#include "scanbusthread.h"

#include <qevent.h>
#include <klocale.h>
#include <kpushbutton.h>
#include <qlabel.h>
#include <kwin.h>
#include <kdebug.h>
#include <kcombobox.h>
#include <kconfig.h>
#include <qstringlist.h>
#include <kdirlister.h>
#include <klistview.h>
#include <kstddirs.h>
#include <qapplication.h>
#include <kmessagebox.h>
#include <qlayout.h>
#include <qsizepolicy.h>
#include <kprocess.h>
#include <qregexp.h>
#include <qtimer.h>
#include <kprogress.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qdir.h>

//taglib includes
#include <fileref.h>
#include <mpegfile.h>
#include <audioproperties.h>
#include <tfile.h>
#include <tstring.h>
#include <flacfile.h>
#include <oggflacfile.h>
#include <vorbisfile.h>

#include <audiofile.h>

BurnDiscWizard::BurnDiscWizard( KConfig *config, QWidget *parent, const char *name)
 : KWizard(parent, name, true),
   m_config( config ),
   m_mediaIsReWritable( false ),
   m_mediaIsDVD( false )
{

  m_dirLister = 0;
  
  if( m_config->readEntry( "file" ) == QString::null )
  {
    //no cd burner has been selected
    m_settingsPage = new DriveSelectPage(m_config, this);
    addPage( m_settingsPage, i18n("Select drive" ) );
    
    setHelpEnabled( m_settingsPage, false );
    setNextEnabled( m_settingsPage, false );
    connect( m_settingsPage->m_listView, SIGNAL( selectionChanged() ),
             m_settingsPage, SLOT(enableNext()) );
    
  }
  else
    m_settingsPage = 0l;
  
  m_burnOptionsPage = new BurnOptionsPage( 0 );
  //m_burnOptionsPage->blankCheckBox->setChecked(m_config->readBoolEntry( "blankcdrw" , false));
  //m_burnOptionsPage->blankOnlyCheckBox->setChecked(m_config->readBoolEntry( "blankonly" , false ));
  m_burnOptionsPage->discTypeGroup->setButton( m_config->readNumEntry( "disctype", 0 ) );

  if( QDir( locateLocal( "data", "kio_burn/data" ) ).count() == 2  ) 
     //if data directory is empty (apart from . and ..), disable data disc button
    m_burnOptionsPage->dataDiscButton->setEnabled( false );  
  if( QDir( locateLocal( "data", "kio_burn/audio" ) ).count() == 2  ) 
    m_burnOptionsPage->audioDiscButton->setEnabled( false );

  connect( m_burnOptionsPage->discTypeGroup, SIGNAL( clicked( int ) ),
           this, SLOT( discTypeChanged( int ) ) );
    
  m_advancedSettingsDialog = new AdvancedSettingsDialog( this, "advancedsettings", true );
  m_advancedSettingsDialog->advancedSettingsCheckBox->setChecked( m_config->readBoolEntry( "advsettings", false ) );
  m_advancedSettingsDialog->noFixateCheckBox->setChecked( m_config->readBoolEntry( "nofix", false ) );
  m_advancedSettingsDialog->burnModeCombo->setCurrentItem( m_config->readNumEntry( "burnmode", 0 ) );
  
  connect( m_burnOptionsPage->advancedOptionsButton, SIGNAL( clicked() ),
           m_advancedSettingsDialog, SLOT( show() ) );
  
  m_playlistPage = new PlaylistPage( 0 );

  addPage( m_burnOptionsPage, i18n( "Burn Options") );
  setHelpEnabled( m_burnOptionsPage, false );
  
  addPage( m_playlistPage, i18n( "Playlist" ));
  setHelpEnabled( m_playlistPage, false );
  if( m_burnOptionsPage->discTypeGroup->selectedId() == 0 ) // data cd
    setAppropriate( m_playlistPage, false );
  
  m_scanPage = new ScanMediaPage();
  m_scanPage->scanProgress->setHidden( true );
  connect( m_scanPage->scanButton, SIGNAL( clicked() ),
           this, SLOT(scanMedia()) );
  
  if( ( m_burnOptionsPage->dataDiscButton->isEnabled() || m_burnOptionsPage->audioDiscButton->isEnabled() ) ) 
    addPage( m_scanPage, i18n( "Checking disc" ) );
  setHelpEnabled( m_scanPage, false );
  
  setCaption( i18n( "Burn disc" ) );
  
  //setFinishEnabled( m_finishPage, true );
  finishButton()->setText( i18n( "Burn disc" ) );
  
  //if the user is creating an audio cd we'll need to scan for files to add to a playlist
  connect( this, SIGNAL( selected(const QString &) ),
           this, SLOT( pageChanged(const QString &) ) );
  
  m_speedProc = 0L;
  m_scannedList = 0L;
  m_outputList = 0L;
  m_savedList = 0L;
}

void BurnDiscWizard::closeEvent( QCloseEvent *e )
{
  e->accept();
  qApp->exit_loop();
}
void BurnDiscWizard::scanMedia()
{
 
  m_scanPage->scanButton->setEnabled( false );
  m_scanPage->scanButton->setText( i18n( "Rescan" ) );
  m_scanPage->scanProgress->setHidden( false );
  
  m_scanPage->instructionLabel->setText( i18n( "Please wait..." ) );
  
  m_scanBusThread = new ScanBusThread( m_config->readEntry( "file" ) );
  m_scanBusThread->start();
  
  m_timer = new QTimer();
  connect( m_timer, SIGNAL( timeout() ),
           this, SLOT( checkScanFinished() ) );
  
  m_timer->start( 500 );
}

void BurnDiscWizard::checkScanFinished()
{
  if( m_scanBusThread->finished() )
  {
    m_scanPage->scanButton->setEnabled( true );
    m_scanPage->scanProgress->setHidden( true );
    
    delete m_timer;
    
    if( m_scanBusThread->currentMedia == ScanBusThread::UNKNOWN )
    {
      m_scanPage->instructionLabel->setText( i18n( "The disc you have inserted was not recognised or no disc is present. Please insert an appropriate writable disc and click the rescan button." ) );
      setFinishEnabled( m_scanPage, false );
      return;
    }
    else
    {
      QString text;
      
      m_mediaIsReWritable = m_scanBusThread->currentMedia == ScanBusThread::CDRW 
          || m_scanBusThread->currentMedia == ScanBusThread::DVDPRW 
          || m_scanBusThread->currentMedia == ScanBusThread::DVDMRWrrec
          || m_scanBusThread->currentMedia == ScanBusThread::DVDMRWrovr
          || m_scanBusThread->currentMedia == ScanBusThread::DVDRAM;
      m_mediaIsDVD =(m_scanBusThread->currentMedia == ScanBusThread::DVDPR
          || m_scanBusThread->currentMedia == ScanBusThread::DVDMR
          || m_scanBusThread->currentMedia == ScanBusThread::DVDMRWrrec
          || m_scanBusThread->currentMedia == ScanBusThread::DVDMRWrovr
          || m_scanBusThread->currentMedia == ScanBusThread::DVDRAM
          || m_scanBusThread->currentMedia == ScanBusThread::DVDPRW);
       
      if( m_burnOptionsPage->discTypeGroup->selectedId() == 1 ) //audio CD selected
      {
        if( !m_mediaIsDVD )
        {
          text = i18n( "Ready to burn an audio CD, click Burn Disc to start the burning process now." ) ;
          
          setFinishEnabled( m_scanPage, true );
        }
        else
        {
          text = i18n( "You have asked to burn an audio CD but you have not inserted a writable CD. Please either insert a writable CD and click rescan, or press back and choose to burn a data disc." ); 
          
          setFinishEnabled( m_scanPage, false );
          
          
          
        }
        
      }
      else 
      { //data CD / DVD selected
        if(m_scanBusThread->currentMedia == ScanBusThread::CDR 
           || m_scanBusThread->currentMedia == ScanBusThread::CDRW )
        {
          text = i18n( "Ready to burn a data CD, click Burn Disc to start the burning process now." );
          
          setFinishEnabled( m_scanPage, true );
        }
        else //all other cases should be DVD
        {
          text = i18n( "Ready to burn a data DVD, click Burn Disc to start the burning process now." );
            
          setFinishEnabled( m_scanPage, true );
        }
      }
      
      if( m_mediaIsReWritable &&  finishButton()->isEnabled() )
      {
        text += i18n( "\n\nWARNING: You are about to burn to a rewritable disc. If you continue, you will erase any data that is already on the disc." );
      }
       
      m_scanPage->instructionLabel->setText( text );
    }
    delete m_scanBusThread;
  } 
}
void BurnDiscWizard::speedProcessExited( KProcess * )
{
  m_advancedSettingsDialog->burnSpeedCombo->removeItem( 0 ); //remove the "scanning..." item
  
  //default to the last used speed, if it is available
  QString speed (  m_config->readEntry( "burnspeed" ) );
  
  m_advancedSettingsDialog->burnSpeedCombo->setCurrentItem( 0 );
    
  if( speed != QString::null )
  {
      for( int i = 0; i <  m_advancedSettingsDialog->burnSpeedCombo->count(); i++ )
    {
      if(m_advancedSettingsDialog->burnSpeedCombo->text( i ) == speed )
      {
        m_advancedSettingsDialog->burnSpeedCombo->setCurrentItem( i );
      }
    }
  }
  delete m_speedProc;
  m_speedProc = 0L;
}


void BurnDiscWizard::speedProcessStdout( KProcess */*proc*/, char *buffer, int buflen )
{
  QString s = QString::fromLatin1( buffer, buflen );
  QStringList l = QStringList::split( '\n', s );
  
  QRegExp rx( "Write speed.*CD *([0-9]+)x");
  
  int z, count = l.count();
  
  for( z = 0; z < count; z++ ) 
  { 
    if( rx.search( l[z] ) > -1 )
    {
      m_advancedSettingsDialog->burnSpeedCombo->insertItem( rx.capturedTexts()[1] );
    }
  }
  
}


void BurnDiscWizard::discTypeChanged( int type )
{

  if( type == 1 )
  {//audio cd
    setAppropriate( m_playlistPage, true );
    
  }
  else
    setAppropriate( m_playlistPage, false );
  
  showPage( m_burnOptionsPage );
  
}

void BurnDiscWizard::pageChanged( const QString & /*title*/ )
{

  if( (currentPage() == m_scanPage) )
  {
    scanMedia();
    if( appropriate( m_playlistPage ) )
    {
      //save the playlist
      QStringList list;
      
      for( QListViewItem *item = m_playlistPage->trackListView->firstChild();
           item != 0L; item = item->itemBelow() )
      {
        list.append( item->text( 4) );
      }
      
      m_config->writeEntry( "tracklist", list );
    }
  }
  if( (currentPage() == m_burnOptionsPage) )
  {
    if( m_settingsPage && (currentPage() == m_burnOptionsPage) )
    {
      m_settingsPage->saveSettings();
    }
    
    //all the setup is done, now scan for available writing speeds
    
    m_speedProc = new KProcess( this, "getspeedproc");
    *m_speedProc << "cdrecord";
    *m_speedProc << "--dev=" + m_config->readEntry( "file" )
        << "-prcap";
      
    connect( m_speedProc, SIGNAL( receivedStdout( KProcess *, char *, int ) ),
            this, SLOT ( speedProcessStdout( KProcess *, char *, int ) ) );
    connect( m_speedProc, SIGNAL( processExited( KProcess * ) ),
            this, SLOT ( speedProcessExited( KProcess * ) ) );
    
    m_speedProc->start(KProcess::NotifyOnExit, KProcess::Stdout);
    
    //write "scanning..." in the speed combo for now
    m_advancedSettingsDialog->burnSpeedCombo->clear();
    m_advancedSettingsDialog->burnSpeedCombo->insertItem( i18n( "Scanning..." ) );
           
  }
   
  if( (currentPage() == m_playlistPage) && (m_playlistPage->trackListView->childCount() == 0) )
  {
    
    disconnect( this, SIGNAL( selected(const QString &) ),
                this, SLOT( pageChanged(const QString &) ) ); //we don't want to do this more than once
    
    m_playlistPage->upButton->setGuiItem( KGuiItem( QString::null, "up" ) );
    m_playlistPage->downButton->setGuiItem( KGuiItem( QString::null, "down") );
    connect( m_playlistPage->upButton, SIGNAL( clicked() ),
             this, SLOT( moveTrackUp() ) );
    connect( m_playlistPage->downButton, SIGNAL( clicked() ),
             this, SLOT( moveTrackDown() ) );
    connect( m_playlistPage->trackListView, SIGNAL( moved() ), this, SLOT( renumberTracks() ) ); 
    
    m_playlistPage->trackListView->setSorting( -1 ) ;
    m_playlistPage->trackListView->setAllColumnsShowFocus( true );
    m_playlistPage->trackListView->setResizePolicy( QScrollView::Manual );
                                                         
    m_scannedList = new QStringList();
    m_outputList = new QStringList();
    m_savedList = new QStringList( m_config->readListEntry( "tracklist" ) );

    if( m_dirLister )
      delete m_dirLister;
    
    m_dirLister = new KDirLister();

    QStringList validMimes;
    validMimes.append( "audio/x-wav" );

    validMimes.append( "audio/x-vorbis" );
    validMimes.append( "audio/vorbis" );
    validMimes.append( "application/ogg" );
    validMimes.append( "application/x-ogg" );
    validMimes.append( "audio/x-mp3");
    //validMimes.append( "audio/x-oggflac");

    m_dirLister->setMimeFilter( validMimes );

    connect( m_dirLister, SIGNAL( newItems( const KFileItemList & ) ), this, SLOT( addItemsToScannedList( const KFileItemList & ) ) );
    connect( m_dirLister, SIGNAL( completed() ), this, SLOT( createTrackListing() ) );

    m_dirLister->openURL( locateLocal( "data", "kio_burn/audio" ), false, true );

       
  }
  
}

void BurnDiscWizard::createTrackListing()
{
  if( m_playlistPage->trackListView->childCount() > 0 )
    return;
  //now we should have a list of files in the audio CD directory in m_scannedList,
    //the tracklisting from the last CD in m_savedList, and we're going to create a
    //new list in m_outputList. This algorithm is a little complex, but the gist is:
    //create a new list that is the same as the saved list, except any files
    //that are in the saved list but not the scanned one should be removed,
    //and any files in the scanned list but not in the saved one should be
    //put at the end.
  QStringList::iterator it_saved, it_scanned, it_output;

  bool matched;

  for ( it_scanned = m_scannedList->begin(); it_scanned != m_scannedList->end(); ++it_scanned )
  {

    matched = false;
    for ( it_saved = m_savedList->begin(); it_saved != m_savedList->end(); ++it_saved )
    {

      if ( *it_scanned == *it_saved )
      {
        matched = true;
        break;
      }

    }

    if ( !matched )
    {

      m_outputList->append( *it_scanned );
      continue;

    }

    if ( it_saved == m_savedList->begin() )
    {
        //ok, we matched the first item in the saved list,
        //it's safe to assume it's got to go at the beginning of the list
      m_outputList->prepend( *it_scanned );
      continue;

    }

    while ( it_saved != m_savedList->begin() )
    {

      --it_saved;

      it_output = m_outputList->begin();
      while ( 1 )
      {

        if ( *it_output == *it_saved )
        {

          m_outputList->insert( ++it_output, *it_scanned );
          goto end_of_loop;

        }

        ++it_output;
        if ( it_output == m_outputList->end() )
          break;
      }

    }


    m_outputList->prepend( *it_scanned );
    end_of_loop: continue;
  }

  addItemsToTrackList( *m_outputList );
   
  connect( this, SIGNAL( selected(const QString &) ),
           this, SLOT( pageChanged(const QString &) ) );

  disconnect( this, SLOT( createTrackListing() ) );
  
  delete m_scannedList;
  delete m_outputList;
  delete m_savedList;
    
  
}
QString numberToQString( int n )
{
  //creates a QString out of an int suitable for the track number column in the display
  //that is - base 10 conversion with a leading 0 for single figure numbers
  
  char str[3];
  
  sprintf( str, "%2d", n );
  
  return QString( str );
  
}

void BurnDiscWizard::renumberTracks()
{

  //change all the track numbers in the list so that they are in ascending order
  
  int i;
  
  QListViewItem *item = m_playlistPage->trackListView->firstChild();
    
  for( i = 1; i <= m_playlistPage->trackListView->childCount(); i++ )
  {
    item->setText( 0, numberToQString( i ) );
    item = item->itemBelow();  
  }
}

void BurnDiscWizard::moveTrackDown()
{

  if( !m_playlistPage->trackListView->childCount() ) return;
  QListViewItem *current = m_playlistPage->trackListView->currentItem();
  QListViewItem *next = current->itemBelow();
  
  if( next == 0L ) return;
  
  QString currentTrackNo = current->text( 0 );
  int trackNo = currentTrackNo.toInt();
  
  current->setText( 0, numberToQString( trackNo + 1 ) );
  next->setText( 0, numberToQString( trackNo ) );
  
  m_playlistPage->trackListView->setSorting( 0 );
  m_playlistPage->trackListView->sort();
  m_playlistPage->trackListView->setSorting( -1 );
  
  m_playlistPage->trackListView->ensureItemVisible( current );
}

void BurnDiscWizard::moveTrackUp()
{

  if( !m_playlistPage->trackListView->childCount() ) return;
  QListViewItem *current = m_playlistPage->trackListView->currentItem();
  QListViewItem *next = current->itemAbove();
  
  if( next == 0L ) return;
  
  QString currentTrackNo = current->text( 0 );
  int trackNo = currentTrackNo.toInt();
  
  current->setText( 0, numberToQString( trackNo - 1 ) );
  next->setText( 0, numberToQString( trackNo ) );
  
  m_playlistPage->trackListView->setSorting( 0 );
  m_playlistPage->trackListView->sort();
  m_playlistPage->trackListView->setSorting( -1 );
  
  m_playlistPage->trackListView->ensureItemVisible( current );
}

QTime timeFromSeconds( int secs )
{
  return QTime( (int)(secs/3600), (int)((secs/60)%60), (int)(secs%60) );
}

void BurnDiscWizard::addItemsToTrackList( const QStringList & items )
{
  
  QStringList::iterator file;
  QStringList fileList = items;
  QString mediaType, *songName = 0;
  KListViewItem *listItem;
  int totalTime = 0;
  int length=0;
  int tracksprocessed = 1;
  KLocale locale( "tmp" );
  int channels=0;
  int sampleRate=0;
  
  for( file = fileList.begin(); file != fileList.end(); file++ )
  {
      
    if( tracksprocessed > 99 )
    {
      KMessageBox::sorry( this, 
                          i18n("You have chosen more than 99 tracks for this CD, but you cannot have this many tracks on a CD. Some of them will be lost") );
      break;
    }
    
    kdDebug() << "Processing " << *file << endl;
    //use taglib to get information about files where possible
    
    KMimeType::Ptr mType =  KMimeType::findByPath( *file,0,true );
    TagLib::FileRef *fileRef = 0L;
    
    if( mType->name() == "audio/x-wav" )
    {
      //wav file - use audiofile to get info

      songName = new QString( *file );
      
      AFfilehandle afFile = afOpenFile( (locateLocal( "data", "kio_burn/audio" ) += QString("/") += *file).local8Bit(), "r", 0L );
      int *format = new int;
    
      if( afGetFileFormat( afFile, format ) != AF_FILE_WAVE ) { //not really a wav file
        //bail out
        kdDebug() << *file << " is not a wav file, skipping" << endl;
        continue;
      }
      
    
      sampleRate = (int)afGetRate( afFile, AF_DEFAULT_TRACK );
      channels = afGetChannels( afFile, AF_DEFAULT_TRACK );
      
      
      AFframecount frameCount = afGetFrameCount( afFile, AF_DEFAULT_TRACK );
      
      afCloseFile( afFile );
      
      length = (int) (frameCount / sampleRate);
      
      mediaType = i18n("Wav audio");

    }
    else if( mType->name() == "audio/x-mp3" )
    {
      
      fileRef = new TagLib::FileRef( new TagLib::MPEG::File( (locateLocal( "data", "kio_burn/audio/" ) += *file).local8Bit(), true, TagLib::AudioProperties::Average) );
      
      mediaType = i18n("MP3 audio");
      
      
    }
    else if( mType->name() == "application/ogg"
             || mType->name() == "application/x-ogg"
             || mType->name() == "audio/vorbis"
             || mType->name() == "audio/x-vorbis")
    {
    
      fileRef = new TagLib::FileRef( new TagLib::Vorbis::File( (locateLocal( "data", "kio_burn/audio/" ) += *file).local8Bit(), true, TagLib::AudioProperties::Average) );
      
      mediaType = i18n("Ogg Vorbis audio");  
      
    }
    /*else if( mType->name() == "audio/x-flac" )
    {
    fileRef = new TagLib::FileRef( new TagLib::FLAC::File( (locateLocal( "data", "kio_burn/audio/" ) += *file).local8Bit(), true, TagLib::AudioProperties::Average) );
      
    mediaType = i18n("FLAC audio"); 
  }*//*else if( mType->name() == "audio/x-oggflac" )
    {
    fileRef = new TagLib::FileRef( new TagLib::FLAC::File( (locateLocal( "data", "kio_burn/audio/" ) += *file).local8Bit(), true, TagLib::AudioProperties::Average) );
    mediaType = i18n("Ogg FLAC audio"); 
                  
  }*/
    else
    {
      //file not a recognised format
      continue;
    }
    
      
    if( fileRef != 0L  )
    { //use taglib to find length
      if( fileRef->file()->isOpen() )
      {
        if( fileRef->audioProperties() == 0L )
        {
          //failed to read audio properties, most likely this is not a valid file
          //of the format we thought it was
          continue;  
        }
        
        length = fileRef->audioProperties()->length();
        if( fileRef->tag()->title() == TagLib::String::null )
          songName = new QString( *file ); //no title in tag - use file name
        else
          songName = new QString( TStringToQString( fileRef->tag()->artist() ) + " - " + TStringToQString( fileRef->tag()->title() ) );
        
        channels = fileRef->audioProperties()->channels();
        sampleRate = fileRef->audioProperties()->sampleRate();
      }
      else
      {
        length = 0;
        songName =  new QString ( *file );
      }
      
    }
    
    if( (sampleRate != 44100.0) || (channels != 2) )
    {
      KMessageBox::error( 0,
                          i18n("The file %1 cannot be used for this CD because it is not a 44.1kHz stereo file. You will need to convert it with an audio editor or conversion utility first").arg( *file ),
                          i18n("Bad file format"));
    }
    
    if( m_playlistPage->trackListView->childCount() )
      totalTime += length;
    else
      totalTime = length;
    
    listItem = new KListViewItem( m_playlistPage->trackListView, numberToQString(m_playlistPage->trackListView->childCount()+1), 
                                  *songName, 
                                  locale.formatTime( timeFromSeconds( length ) , true, true ),
                                  mediaType,
                                  *file) ;
    tracksprocessed++;
  }
  
  m_playlistPage->trackListView->setSorting( 0 );
  m_playlistPage->trackListView->sort();
  m_playlistPage->trackListView->setSorting( -1 );

  m_playlistPage->totalPlayingTimeLabel->setText( QString( "<p align=\"right\">" )
      += i18n( "Total playing time: %1").arg( locale.formatTime( timeFromSeconds( totalTime ), true, true ) )
      += QString("</p>"));
}

void BurnDiscWizard::exitQAppLoop() const
{
 
  qApp->exit_loop();
   
}

void BurnDiscWizard::addItemsToScannedList( const KFileItemList & items )
{

  KFileItem * item;
  KFileItemList itemList = items;

  for ( item = itemList.first(); item; item = itemList.next() )
  {
    m_scannedList->append( item->name() );
  }
}



BurnDiscWizard::~BurnDiscWizard()
{
  disconnect( this, SIGNAL( selected(const QString &) ),
           this, SLOT( pageChanged(const QString &) ) );
  if( m_speedProc )
    delete m_speedProc;
  delete m_burnOptionsPage;
  delete m_advancedSettingsDialog;
  delete m_playlistPage;
  delete m_settingsPage;
  delete m_scanPage;
  if( m_dirLister )
    delete m_dirLister;
  
}
#include "burndiscwizard.moc"

