/***************************************************************************
 *   Copyright (C) 2004 by James Thorniley                                 *
 *   james@uncommonlygood.co.uk                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "driveselectpage.h"
#include "scanbusthread.h"

#include <qlayout.h>
#include <klistview.h>
#include <qlabel.h>
#include <qfileinfo.h>
#include <klocale.h>
#include <kprocess.h>
#include <qapplication.h>
#include <qregexp.h>
#include <kdebug.h>
#include <kprogress.h>
#include <kconfig.h>
#include <qtimer.h>
#include <qwhatsthis.h>
#include <qpushbutton.h>
#include <qvaluelist.h>
#include <klineedit.h>
#include <kurlcompletion.h>

DriveSelectPage::DriveSelectPage(KConfig *config, KWizard *parentWizard, QWidget *parent, const char *name)
 : QWidget(parent, name)
{
  m_config = config;

  this->setMinimumHeight( 200 );
  this->setMinimumWidth( 200 );

  m_instructions1 = new QLabel( this, 0);
  m_instructions1->setText(i18n("Please wait while KDE scans for CD/DVD devices") );
  m_instructions1->setAlignment( int( QLabel::WordBreak | QLabel::AlignVCenter ) );
    
  m_listView = new KListView( this);
  m_listView->addColumn( i18n("Device name"));
  m_listView->addColumn( i18n("Type") );
  m_listView->addColumn( i18n("File name" ) );
  m_listView->setAllColumnsShowFocus( true );
  m_listView->setSelectionMode( QListView::Single );
  m_listView->setSorting( -1 );
  m_listView->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Expanding );
  
  m_progress = new KProgress( this );
  m_progress->setTotalSteps( 100 );
  m_progress->setProgress( 99 );
  m_progress->setTextEnabled( false );
  
#ifndef HAVE_K3B
  m_instructions2 = new QLabel( this, 0 );
  m_instructions2->setText( i18n("If your CD or DVD burner is not listed above, please enter the file name below") );

  m_instructions2->setAlignment( int( QLabel::WordBreak | QLabel::AlignVCenter ) );
  
  m_fileInput = new KLineEdit( this, 0 );
  m_fileInput->setCompletionObject( new KURLCompletion() );
  m_fileInput->setAutoDeleteCompletionObject( true );
  connect( m_fileInput, SIGNAL( textChanged( const QString & ) ),
           this, SLOT( slotFileInputChanged( const QString & ) ) );
#endif

  m_layout = new QVBoxLayout( this,5 );
  m_layout->addStretch();
  m_layout->addWidget( m_instructions1 );
  m_layout->addWidget( m_progress );
  m_layout->addWidget( m_listView );
  
#ifndef HAVE_K3B
  m_layout->addWidget( m_instructions2 );
  m_layout->addWidget( m_fileInput );
#endif

  m_layout->addStretch();
  
  QSize widgetSize = sizeHint();
  widgetSize.setHeight( (int)((widgetSize.height() - m_progress->sizeHint().height()) * 0.8) );
  widgetSize.setWidth( (int)(widgetSize.width() * 1.6) );  //the width will need some spare
  
  setMinimumSize( widgetSize ); //first make sure our minimum size is big enough to accommodate averything
  
  m_listView->setHidden(true); //then hide everything we won't need until the scan is complete
#ifndef HAVE_K3B
  m_instructions2->setHidden( true );
  m_fileInput->setHidden( true );
#endif
  QWhatsThis::add( m_listView, i18n( "<p>Select a device from this list.<p><b>Device name</b> is the description this device reported for itself.<p><b>Type</b> specifies whether this is a CD or DVD burner.<p><b>File name</b> is the UNIX file name associated with this device. If you don't know what this means, don't worry. It's not important unless you have two identical devices and you need to distinguish between them, in which case you'll have to find out somehow which device corresponds to which file name." ) );
  
  m_wizard = parentWizard;
  
  m_timer = new QTimer();
  connect( m_timer, SIGNAL( timeout() ),
           this, SLOT( checkScanFinished() ) );
  m_timer->start( 1000 );
  
  m_scanBusThread = new ScanBusThread();
  m_scanBusThread->start();
}


void DriveSelectPage::saveSettings()
{
  int i;
  for( i = 0; m_scanBusThread->deviceList()[i].fileName != m_listView->currentItem()->text(2); i++ ); 
  
#ifndef HAVE_K3B
  if( m_listView->isEnabled() )
    m_config->writeEntry( "file", m_scanBusThread->deviceList()[i].fileName );
  else
    m_config->writeEntry( "file", m_fileInput->text() );
#else
  m_config->writeEntry( "file", m_scanBusThread->deviceList()[i].fileName );
#endif
  m_config->writeEntry( "cdr", m_scanBusThread->deviceList()[i].CDR );
  m_config->writeEntry( "cdrw", m_scanBusThread->deviceList()[i].CDRW );
  m_config->writeEntry( "dvd-r", m_scanBusThread->deviceList()[i].DVDMR );
  m_config->writeEntry( "dvd-rwresovr", m_scanBusThread->deviceList()[i].DVDMRWrovr );
  m_config->writeEntry( "dvd-rwresrec", m_scanBusThread->deviceList()[i].DVDMRWrrec );
  m_config->writeEntry( "dvd-ram", m_scanBusThread->deviceList()[i].DVDRAM );
  m_config->writeEntry( "dvd+r", m_scanBusThread->deviceList()[i].DVDPR );
  m_config->writeEntry( "dvd+rw", m_scanBusThread->deviceList()[i].DVDPRW );
  m_config->writeEntry( "isdvdr", m_scanBusThread->deviceList()[i].isDVDR );
  m_config->sync();
  
}
DriveSelectPage::~DriveSelectPage()
{
  delete m_instructions1;
  delete m_progress;
  delete m_layout;
  delete m_listView;
  delete m_scanBusThread;
  
#ifndef HAVE_K3B
  delete m_instructions2;
  delete m_fileInput;
#endif
}

void DriveSelectPage::checkScanFinished()
{
  if( m_scanBusThread->finished() )
  {
    delete m_timer;
    
    m_instructions1->setText( i18n( "Please choose the device you would like to use from the list below" ) );
    m_progress->setHidden( true );
    m_listView->setHidden( false );
#ifndef HAVE_K3B
    m_instructions2->setHidden( false );
    m_fileInput->setHidden( false );
#endif
    
    QValueList<Device>::Iterator dev;
    for( dev = m_scanBusThread->deviceList().begin(); dev != m_scanBusThread->deviceList().end(); dev++ )
    {     
      if( (*dev).CDR | (*dev).isDVDR )
        new KListViewItem( m_listView,
                           (*dev).desc,
                           (*dev).isDVDR ? i18n( "DVD burner" ) : i18n( "CD burner" ),
                           (*dev).fileName);
  
      
    }
  }
}
#include "driveselectpage.moc"


/*!
    \fn DriveSelectPage::slotFileInputChanged( const QString &text )
    
    Makes sure the scanned device list is disabled when the manual filename input box
    is being used.

 */
 
void DriveSelectPage::slotFileInputChanged( const QString &text )
{
  kdDebug() << "void DriveSelectPage::slotFileInputChanged( const QString &text = " << text << " )";
  
  if( text.isEmpty() )
  {
    m_listView->setEnabled( true );
    
    if( m_listView->selectedItem() )
      m_wizard->setFinishEnabled( this, true );
    else
      m_wizard->setFinishEnabled( this, false );
  }
  else
  {
    m_listView->setEnabled( false );
    
    QFileInfo f( text );
    if( f.exists() && !f.isDir() )
      m_wizard->setFinishEnabled( this, true );
    else
      m_wizard->setFinishEnabled( this, false );
  }
  
  
}

