/***************************************************************************
 *   Copyright (C) 2004 by James Thorniley                                 *
 *   james@uncommonlygood.co.uk                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kio_burnplugin.h"
#include "burn.h"
#include "progressdlg.h"

#include <klocale.h>
#include <kio/global.h>
#include <kio/job.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <dcopclient.h>
#include <dcopobject.h>
#include <kxmlguiclient.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <qlabel.h>
#include <kdialogbase.h>

kio_burnPlugin::kio_burnPlugin(QObject *parent, const char *name)
  : KParts::Plugin(parent, name), DCOPObject( "kio_burnPlugin" )
{

  (void) new KAction( i18n( "Burn Disc" ), 
                      "cdwriter_unmount", 
                      0,
                      this, SLOT( startBurn() ),
                      actionCollection(), "burndisc" );
  
  m_progress = new ProgressDlg ( qApp->mainWidget(), "burnprogress", false, Qt::WType_Dialog || Qt::WStyle_DialogBorder || Qt::WStyle_Dialog );

}

void kio_burnPlugin::startBurn()
{
  KURL url;
  url.setProtocol( "burn" );
  url.setPath( QString("/").append( i18n("Burn Disc") ) );
  url.setQuery( QString("?dcopid=").append( KApplication::dcopClient()->appId() ) );
  static KIO::Job *burnJob;
  burnJob = KIO::get( url , false , false);
  connect( burnJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( burnResult( KIO::Job * ) ) );
  kdDebug(0) << "text" << endl;
}

kio_burnPlugin::~kio_burnPlugin()
{
  delete m_progress;
}

void kio_burnPlugin::burnResult( KIO::Job *job )
{

  if( job->error() ) //this doesn't seem to work, oh well
  {
    job->showErrorDialog();
  }  
}

void kio_burnPlugin::showProgressBar( bool show ){
 
  if( show ) 
  {
    m_progress->show();
  }else
  {
    m_progress->hide();
  }
  
}

void kio_burnPlugin::setProgressBarText(QString text )
{
  m_progress->textLabel->setText( text );
}

void kio_burnPlugin::setProgressBarValue( int value )
{
  m_progress->progressBar->setProgress( value );
}

void kio_burnPlugin::setProgressBarTotal( int total )
{
  m_progress->progressBar->setTotalSteps( total );
}
  
KPluginFactory::KPluginFactory( QObject* parent, const char* name )
  : KLibFactory( parent, name )
{
  s_instance = new KInstance("KPluginFactory");
}

QObject* KPluginFactory::createObject( QObject* parent, const char* name,
                                 const char*, const QStringList & )
{
  QObject *obj = new kio_burnPlugin( parent, name );
  emit objectCreated( obj );
  return obj;
}

extern "C"
{
  void *init_kio_burn()
  {
    KGlobal::locale()->insertCatalogue( "kio_burn" );
    return new KPluginFactory;
  }
}

KInstance *KPluginFactory::s_instance = 0L;

#include "kio_burnplugin.moc"
