/***************************************************************************
 *   Copyright (C) 2005 by James Thorniley                                 *
 *   james@uncommonlygood.co.uk                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "scanbusthread.h"

#undef __STRICT_ANSI__
#include "transport.h"

#include <qstringlist.h>
#include <kdebug.h>

#ifdef HAVE_K3B
#include <k3bdevicemanager.h>
#include <k3bdevice.h>
using namespace K3bCdDevice;

#endif

#define DEVICE_LIST "sr0 sr1 sr2 sr3 sr4 sr5 sr6 sr7 sr8 sr9 sr10 sr11 sr12 sr13 sr14 sr15 hda hdb hdc hdd hde hdf hdg hdh hdi hdj hdk hdl hdm hdn hdo hdp hdq hdr hds hdt"

ScanBusThread::ScanBusThread()
 : QThread()
{
  m_scanFileOnly = false;
  m_fileName = 0;
#ifdef HAVE_K3B
  m_deviceManager = new DeviceManager(0,0);
#endif
}

ScanBusThread::ScanBusThread( const QString & fileName )
{
  m_scanFileOnly = true;
  m_fileName = new QString( fileName ); 
#ifdef HAVE_K3B
  m_deviceManager = new DeviceManager(0,0);
#endif
}

void ScanBusThread::run()
{
 
  if( m_scanFileOnly )
  { //set currentMedia to the type that most closely matches what is in the drive
    
#ifdef HAVE_K3B
    m_deviceManager->scanbus();
    CdDevice *k3bDevice = m_deviceManager->findDevice( *m_fileName );
    
    if( !k3bDevice ) 
    {
      currentMedia = UNKNOWN;
      return;
    }
    
    int type = 0;
    if( k3bDevice->isDVD() )
      type = k3bDevice->dvdMediaType();
    else
      type = k3bDevice->cdMediaType();
    
    if( type == MEDIA_CD_R )
      currentMedia = CDR;
    else if( type == MEDIA_CD_RW )
      currentMedia = CDRW;
    else if( type == MEDIA_DVD_R || type == MEDIA_DVD_R_SEQ )
      currentMedia = DVDMR;
    else if( type == MEDIA_DVD_RAM )
      currentMedia = DVDRAM;
    else if( type == MEDIA_DVD_RW || type == MEDIA_DVD_RW_SEQ )
      currentMedia = DVDMRWrrec;
    else if( type == MEDIA_DVD_PLUS_R || type == MEDIA_DVD_PLUS_R_DL )
      currentMedia = DVDPR;
    else if( type == MEDIA_DVD_PLUS_RW )
      currentMedia = DVDPRW;
    else
      currentMedia = UNKNOWN;  
    
#else

    Scsi_Command *cmd = new Scsi_Command();
    char *resp = new char[8];
    
    if( !cmd->associate( m_fileName->local8Bit() ) )
      return; //this is unlikely
    
    (*cmd)[0] = 0x46; //get config
    (*cmd)[1] = 1;    //get current config
    (*cmd)[8] = 8;
    
    if( cmd->transport( READ, resp, 8 ) )
      return;
    
    switch( ((int)resp[6] << 8) + (int)resp[7] )
    {
      case 0x9:
        currentMedia = CDR;
        break;
      case 0xa:
        currentMedia = CDRW;
        break;
      case 0x11:
        currentMedia = DVDMR;
        break;
      case 0x12:
        currentMedia = DVDRAM;
        break;
      case 0x13:
        currentMedia = DVDMRWrovr;
        break;
      case 0x14:
        currentMedia = DVDMRWrrec;
        break;
      case 0x1a:
        currentMedia = DVDPRW;
        break;
      case 0x1b:
        currentMedia = DVDPR;
        break;
      default:
        currentMedia = UNKNOWN;
        break;
    }
    delete cmd;
#endif
  }
  else
  //find possible devices and descriptions for them
  {
    //whether we use k3b or not, for each device found, we will
    //create a new Device (defined in scanbusthread.h), and then append
    //it to m_deviceList; 
#ifdef HAVE_K3B

    //use k3bdevicemanager
    m_deviceManager->scanbus();
    
    QPtrList<CdDevice> devList( m_deviceManager->allDevices() );
    
    kdDebug() << "Scanning with K3B" ;
    
    for( CdDevice *k3bDevice = devList.first(); k3bDevice; k3bDevice = devList.next() )
    {
      
      Device *d = new Device;
      
      d->CDR = k3bDevice->writesCd();
      d->CDRW = k3bDevice->writesCdrw();
      d->DVDMR = k3bDevice->type() & CdDevice::DVDR;
      d->DVDRAM = k3bDevice->type() & CdDevice::DVDRAM;
      d->DVDMRWrovr = k3bDevice->type() & CdDevice::DVDRW;
      d->DVDMRWrrec = d->DVDMRWrovr; 
      d->DVDPR = k3bDevice->type() & CdDevice::DVDPR;
      d->DVDPRW = k3bDevice->type() & CdDevice::DVDPRW;
      
      d->desc = (QString( k3bDevice->vendor() ) + QString( k3bDevice->description() )).ascii();
      d->isDVDR = d->DVDMR || d->DVDPR;
      d->fileName = k3bDevice->devicename();
      
      m_deviceList.append( *d );
      
      delete d;
    }
    
#else
  QStringList scanList = QStringList::split( " ",DEVICE_LIST);
  QStringList::iterator file;
  
  Scsi_Command *cmd = new Scsi_Command();
  char inq[36], config[8192];
  for( file = scanList.begin(); file != scanList.end(); ++file )
  {
    if( !cmd->associate( (QString("/dev/") + *file).local8Bit() ) )
      continue;
    (*cmd)[0] = 0x12;
    (*cmd)[4] = 36;
    (*cmd)[5] = 0;
    
    if( cmd->transport(READ,inq,36) )
      continue;
    
    if ((inq[0]&0x1F) != 5)
      continue; //this is not an mmc unit

    char *buf = new char[25];
    snprintf( buf, 24, "%.24s", inq+8 );
    QString *description = new QString( buf );
   
    Device *d = new Device;
    
    d->CDR = false;
    d->CDRW = false;
    d->DVDMR = false;
    d->DVDRAM = false;
    d->DVDMRWrovr = false;
    d->DVDMRWrrec = false;
    d->DVDPRW = false;
    d->DVDPR = false;
    
    (*cmd)[0] = 0x46; //get config
    (*cmd)[1] = 1;    //get current config
    (*cmd)[7] = 0x20; //cmd[7] and cmd[8] give length (8192d)
 
    if ( cmd->transport(READ,config,8192) )
      continue;

    delete buf;
 
    buf = config + 8;
    int buflen = 8192-8;
    int off;
    while( (((int)buf[0] << 8) + (int)buf[1]) != 0 )
    {
      if( buflen < 4 ) break;
      off= 4 + buf[3];
      if( buflen < off ) break;
      buf += off;
      buflen -= off;
    }
    off= 4 + buf[3];
    
    
    for( int i = 4; i < off; i += 4 )
    {
      switch( ((int)buf[i] << 8) + (int)buf[i+1] )
      {
        case 0x8:
          break;
        case 0x9:
          d->CDR = true;
          break;
        case 0xa:
          d->CDRW = true;
          break;
        case 0x11:
          d->DVDMR = true;
          break;
        case 0x12:
          d->DVDRAM = true;
          break;
        case 0x13:
          d->DVDMRWrovr = true;
          break;
        case 0x14:
          d->DVDMRWrrec = true;
          break;
        case 0x1a:
          d->DVDPRW = true;
          break;
        case 0x1b:
          d->DVDPR = true;
          break;
      }
    }
    d->isDVDR = d->DVDMR || d->DVDPR; //these will be worked out later
    d->desc = *description;
    d->fileName = QString("/dev/") + *file;
    
    m_deviceList.append(*d);

    delete description;
    delete d;
    
  }
  delete cmd; 
#endif
  }
}
ScanBusThread::~ScanBusThread()
{
  if( m_fileName )
    delete m_fileName;
#ifdef HAVE_K3B
  delete m_deviceManager;
#endif
}


