/***************************************************************************
 *   Copyright (C) 2005 by James Thorniley                                 *
 *   james@uncommonlygood.co.uk                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SCANBUSTHREAD_H
#define SCANBUSTHREAD_H

#include <qthread.h>
#include <qvaluelist.h>
#include "config.h"

#ifdef HAVE_K3B
namespace K3bCdDevice{
  class DeviceManager;
}

using namespace K3bCdDevice;
#endif

typedef struct Device {
  bool isCDR, isDVDR;
  bool CDR, CDRW, DVDMR, DVDRAM, DVDMRWrovr, DVDMRWrrec, DVDPRW, DVDPR;
  QString desc, fileName;
};
    
/**\brief Class for scanning devices attached to the system
@par
This class is designed to be run as a thread to scan for CD/DVD burners attached to the
system, and can be used to query a drive as to the type of media currently inserted.

@par
An object should be instantiated with ScanBusThread()
if you want to scan the entire
bus for devices. Call the start() method on the object, and when this is finished (check using QThread::finished())
you can call deviceList() to get a list of devices found on the system.

@par
If instantiated with a file name then will set the value of currentMedia to the type of media
inserted.

@see currentMedia

@author James Thorniley
*/
class ScanBusThread : public QThread
{
  public:
    /**Create a thread to scan the entire bus for available CD/DVD burning devices.*/
    ScanBusThread(); 
    /**Create a thread to scan for the type of media inserted in the given
    disc drive
  
    @param fileName The UNIX filename of the device to scan (e.g. "/dev/cdrom")*/
    ScanBusThread( const QString & fileName ); 

    ~ScanBusThread();

    /**Returns a list of Devices that have been found by the scan bus when in bus scanning mode
    @see ScanBusThread()*/
    QValueList<Device> & deviceList() { return m_deviceList; }
    
    /**Possible disc types.*/
    enum media { CDR, CDRW, DVDMR, DVDMRWrovr, DVDMRWrrec, DVDRAM, DVDPR, DVDPRW, UNKNOWN } currentMedia ; 
    /**<The type of disc found in the drive. Note this is only available in media scan mode
    @see ScanBusThread( const QString& fileName )*/
    
  protected:
    virtual void run();
  private:
    QValueList<Device> m_deviceList;
    QString *m_fileName;
    bool m_scanFileOnly;

#ifdef HAVE_K3B
    DeviceManager *m_deviceManager;
#endif
};

#endif
